/***********************************************************************
This is for dealing with the 16x2 LCD. The lower level functions were 
taken from the Microchip C18 Periphreal Library. They were copied and
renamed in this file because the Periphreal library requires that you
recompile it seperately from your project when you change the header
file which contains the pin defentitions. There is also a different 
version for extended instructions that must be recompiled. It started 
to get confusing when I had different projects that used the library 
that had different pin assignments. The solution for me was just to 
duplicate and rename the functions I used in each project. 
**********************************************************************/



#define __16X2LCD_C

#include "TCPIP Stack/TCPIP.h"

#if defined(USE_LCD)

#include "lcd.h"
#include <delays.h>




void DelayFor18TCY( void )
{
	Nop();
	Nop();
	Nop();
	Nop();
	Nop();
	Nop();
	Nop();
	Nop();
	Nop();
	Nop();
	Nop();
	Nop();
}
void DelayPORLCD (void)
{
	Delay10KTCYx(30); // Delay of 15ms
	// Cycles = (TimeDelay * Fosc) / 4
	// Cycles = (15ms * 8MHz) / 4
	// Cycles = 30,000
	return;
}
void DelayLCD (void)
{
	Delay10KTCYx(10); // Delay of 5ms
	// Cycles = (TimeDelay * Fosc) / 4
	// Cycles = (5ms * 8MHz) / 4
	// Cycles = 10,000
	return;
}




#define FOUR_BIT_MODE
//#define SAMSUNG_S6A0032		// This LCD driver chip has a different means of entering 4-bit mode.  

// LCDText is a 32 byte shadow of the LCD text.  Write to it and 
// then call LCDUpdate() to copy the string into the LCD module.
BYTE LCDText[16*2+1];


static void LCDWrite(BYTE RS, BYTE Data)
{
	if (RS)
	{
		WriteDataLCD(Data);
	}
	else
	{
		WriteCmdLCD(Data);
	}
}	


void LCDInit(void)
{
	BYTE i;
	DelayMs(200);
	OpenLCD(FOUR_BIT & LINES_5X7);
	while(BusyLCD());  
	WriteCmdLCD(0x08);
	while(BusyLCD());  
	WriteCmdLCD(0x0c);
	while(BusyLCD());  
	WriteCmdLCD(0x01);
	while(BusyLCD());  
	WriteCmdLCD(0x06);
	while(BusyLCD());   
	
	/*Put wireless bar into CGRAM*/
	
	for (i=1;i<=7;i++)
	{
		BYTE t;
		LCDWrite(0, (i * 8) + 0x40);
		while(BusyLCD()); 
		for (t=0;t<8;t++)
		{
			if (t >= (8-i))
			{
				LCDWrite(1, 0xE);	
			} else
			{
				LCDWrite(1, 0);
			}	
			while(BusyLCD()); 
		}	
	}
	// Go home
	LCDWrite(0, 0x02);
	


}	

/******************************************************************************
 * Function:        void LCDUpdate(void)
 *
 * PreCondition:    LCDInit() must have been called once
 *
 * Input:           LCDText[]
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        Copies the contents of the local LCDText[] array into the 
 *					LCD's internal display buffer.  Null terminators in 
 *					LCDText[] terminate the current line, so strings may be 
 *					printed directly to LCDText[].
 *
 * Note:            None
 *****************************************************************************/
void LCDUpdate(void)
{
	BYTE i, j;

	// Go home
	LCDWrite(0, 0x02);
	DelayMs(2);
	// Output first line
	for(i = 0; i < 16u; i++)
	{
		// Erase the rest of the line if a null char is 
		// encountered (good for printing strings directly)
		
		if(LCDText[i] == 0u)
		{
			for(j=i; j < 16u; j++)
			{
				LCDText[j] = ' ';
			}
		}
		LCDWrite(1, LCDText[i]);
		Delay10us(5);
	}
	
	// Set the address to the second line
	LCDWrite(0, 0xC0);
	Delay10us(5);

	// Output second line
	for(i = 16; i < 32u; i++)
	{
		// Erase the rest of the line if a null char is 
		// encountered (good for printing strings directly)
		if(LCDText[i] == 0u)
		{
			for(j=i; j < 32u; j++)
			{
				LCDText[j] = ' ';
			}
		}
		LCDWrite(1, LCDText[i]);
		Delay10us(5);
	}
}

/******************************************************************************
 * Function:        void LCDErase(void)
 *
 * PreCondition:    LCDInit() must have been called once
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        Clears LCDText[] and the LCD's internal display buffer
 *
 * Note:            None
 *****************************************************************************/
void LCDErase(void)
{
	// Clear display
	LCDWrite(0, 0x01);
	DelayMs(2);

	// Clear local copy
	memset(LCDText, ' ', 32);
}

#endif	//#if defined(USE_LCD)

void WriteCmdLCD(unsigned char cmd)
{
#ifdef BIT8                             // 8-bit interface
        TRIS_DATA_PORT = 0;             // Data port output
        DATA_PORT = cmd;                // Write command to data port
        RW_PIN = 0;                     // Set the control signals
        RS_PIN = 0;                     // for sending a command
        DelayFor18TCY();
        E_PIN = 1;                      // Clock the command in
        DelayFor18TCY();
        E_PIN = 0;
        DelayFor18TCY();
        TRIS_DATA_PORT = 0xff;          // Data port input
#else                                   // 4-bit interface
		#ifdef UPPER                            // Upper nibble interface
		        TRIS_DATA_PORT &= 0x0f;
		        DATA_PORT &= 0x0f;
		        DATA_PORT |= cmd&0xf0;
		#else                                   // Lower nibble interface
		        TRIS_DATA_PORT &= 0xf0;
		        DATA_PORT &= 0xf0;
		        DATA_PORT |= (cmd>>4)&0x0f;
		#endif
        RW_PIN = 0;                     // Set control signals for command
        RS_PIN = 0;
        DelayFor18TCY();
        E_PIN = 1;                      // Clock command in
        DelayFor18TCY();
        E_PIN = 0;
		#ifdef UPPER                            // Upper nibble interface
		        DATA_PORT &= 0x0f;
		        DATA_PORT |= (cmd<<4)&0xf0;
		#else                                   // Lower nibble interface
		        DATA_PORT &= 0xf0;
		        DATA_PORT |= cmd&0x0f;
		#endif
        DelayFor18TCY();
        E_PIN = 1;                      // Clock command in
        DelayFor18TCY();
        E_PIN = 0;
		#ifdef UPPER                            // Make data nibble input
		        TRIS_DATA_PORT |= 0xf0;
		#else
		        TRIS_DATA_PORT |= 0x0f;
		#endif
#endif
        return;
}

void WriteDataLCD(char data)
{
#ifdef BIT8                             // 8-bit interface
        TRIS_DATA_PORT = 0;             // Make port output
        DATA_PORT = data;               // Write data to port
        RS_PIN = 1;                     // Set control bits
        RW_PIN = 0;
        DelayFor18TCY();
        E_PIN = 1;                      // Clock data into LCD
        DelayFor18TCY();
        E_PIN = 0;
        RS_PIN = 0;                     // Reset control bits
        TRIS_DATA_PORT = 0xff;          // Make port input
#else                                   // 4-bit interface
		#ifdef UPPER                            // Upper nibble interface
		        TRIS_DATA_PORT &= 0x0f;
		        DATA_PORT &= 0x0f;
		        DATA_PORT |= data&0xf0;
		#else                                   // Lower nibble interface
		        TRIS_DATA_PORT &= 0xf0;
		        DATA_PORT &= 0xf0;
		        DATA_PORT |= ((data>>4)&0x0f);
		#endif
        RS_PIN = 1;                     // Set control bits
        RW_PIN = 0;
        DelayFor18TCY();
        E_PIN = 1;                      // Clock nibble into LCD
        DelayFor18TCY();
        E_PIN = 0;
		#ifdef UPPER                            // Upper nibble interface
		        DATA_PORT &= 0x0f;
		        DATA_PORT |= ((data<<4)&0xf0);
		#else                                   // Lower nibble interface
		        DATA_PORT &= 0xf0;
		        DATA_PORT |= (data&0x0f);
		#endif
        DelayFor18TCY();
        E_PIN = 1;                      // Clock nibble into LCD
        DelayFor18TCY();
        E_PIN = 0;
		#ifdef UPPER                            // Upper nibble interface
		        TRIS_DATA_PORT |= 0xf0;
		#else                                   // Lower nibble interface
		        TRIS_DATA_PORT |= 0x0f;
		#endif
#endif
        return;
}

/********************************************************************
*       Function Name:  OpenLCD                                    *
*       Return Value:   void                                        *
*       Parameters:     lcdtype: sets the type of LCD (lines)       *
*       Description:    This routine configures the LCD. Based on   *
*                       the Hitachi HD44780 LCD controller. The     *
*                       routine will configure the I/O pins of the  *
*                       microcontroller, setup the LCD for 4- or    *
*                       8-bit mode and clear the display. The user  *
*                       must provide three delay routines:          *
*                       DelayFor18TCY() provides a 18 Tcy delay     *
*                       DelayPORXLCD() provides at least 15ms delay *
*                       DelayXLCD() provides at least 5ms delay     *
********************************************************************/
void OpenLCD(unsigned char lcdtype)
{
        // The data bits must be either a 8-bit port or the upper or
        // lower 4-bits of a port. These pins are made into inputs
#ifdef BIT8                             // 8-bit mode, use whole port
        DATA_PORT = 0;
        TRIS_DATA_PORT = 0x00;
#else                                   // 4-bit mode
#ifdef UPPER                            // Upper 4-bits of the port
        DATA_PORT &= 0x0f;
        TRIS_DATA_PORT &= 0x0F;
#else                                   // Lower 4-bits of the port
        DATA_PORT &= 0xf0;
        TRIS_DATA_PORT &= 0xF0;
#endif
#endif
        TRIS_RW = 0;                    // All control signals made outputs
        TRIS_RS = 0;
        TRIS_E = 0;
        RW_PIN = 0;                     // R/W pin made low
        RS_PIN = 0;                     // Register select pin made low
        E_PIN = 0;                      // Clock pin made low

        // Delay for 15ms to allow for LCD Power on reset
        DelayPORLCD();
 //-------------------reset procedure through software----------------------       
		 WriteCmdLCD(0x30);
			Delay10KTCYx(0x05);

		 WriteCmdLCD(0x30);
			Delay10KTCYx(0x01);


		 WriteCmdLCD(0x32);
		while( BusyLCD() );
//------------------------------------------------------------------------------------------


        // Set data interface width, # lines, font
        while(BusyLCD());              // Wait if LCD busy
        WriteCmdLCD(lcdtype);          // Function set cmd

        // Turn the display on then off
        while(BusyLCD());              // Wait if LCD busy
        WriteCmdLCD(DOFF&CURSOR_OFF&BLINK_OFF);        // Display OFF/Blink OFF
        while(BusyLCD());              // Wait if LCD busy
        WriteCmdLCD(DON&CURSOR_ON&BLINK_ON);           // Display ON/Blink ON

        // Clear display
        while(BusyLCD());              // Wait if LCD busy
        WriteCmdLCD(0x01);             // Clear display

        // Set entry mode inc, no shift
        while(BusyLCD());              // Wait if LCD busy
        WriteCmdLCD(SHIFT_CUR_LEFT);   // Entry Mode

        // Set DD Ram address to 0
        while(BusyLCD());              // Wait if LCD busy
        SetDDRamAddr(0x80);                // Set Display data ram address to 0

        return;
}

/********************************************************************
*       Function Name:  BusyLCD                                    *
*       Return Value:   char: busy status of LCD controller         *
*       Parameters:     void                                        *
*       Description:    This routine reads the busy status of the   *
*                       Hitachi HD44780 LCD controller.             *
********************************************************************/
unsigned char BusyLCD(void)
{
        RW_PIN = 1;                     // Set the control bits for read
        RS_PIN = 0;
        DelayFor18TCY();
        E_PIN = 1;                      // Clock in the command
        DelayFor18TCY();
#ifdef BIT8                             // 8-bit interface
        if(DATA_PORT&0x80)                      // Read bit 7 (busy bit)
        {                               // If high
                E_PIN = 0;              // Reset clock line
                RW_PIN = 0;             // Reset control line
                return 1;               // Return TRUE
        }
        else                            // Bit 7 low
        {
                E_PIN = 0;              // Reset clock line
                RW_PIN = 0;             // Reset control line
                return 0;               // Return FALSE
        }
#else                                   // 4-bit interface
		#ifdef UPPER                            // Upper nibble interface
		        if(DATA_PORT&0x80)
		#else                                   // Lower nibble interface
		        if(DATA_PORT&0x08)
		#endif
        {
                E_PIN = 0;              // Reset clock line
                DelayFor18TCY();
                E_PIN = 1;              // Clock out other nibble
                DelayFor18TCY();
                E_PIN = 0;
                RW_PIN = 0;             // Reset control line
                return 1;               // Return TRUE
        }
        else                            // Busy bit is low
        {
                E_PIN = 0;              // Reset clock line
                DelayFor18TCY();
                E_PIN = 1;              // Clock out other nibble
                DelayFor18TCY();
                E_PIN = 0;
                RW_PIN = 0;             // Reset control line
                return 0;               // Return FALSE
        }
#endif
}
