/*********************************************************************
 This file monitors the TCP port and entertains incoming commands. The
 code here is based from the Berkley Server example inclused with the
 Microchip TCP/IP stack.
 ********************************************************************/
 
#include "TCPIPConfig.h" 
#include "I2CComm.h"
#include "LEDCommands.h"

#include "TCPIP Stack/TCPIP.h"


#define PORTNUM 4764
#define MAX_CLIENT (3) // Maximum number of simultanous connections accepted by the server.

extern LED_TABLE LEDTable[LED_TABLE_LENGTH];
extern BYTE ActiveLEDs;
extern int VoltageMeasurement;
extern int CurrentMeasurement;




/*********************************************************************
 * Function:        void BerkeleyTCPServerDemo(void)
 *
 * PreCondition:    Stack is initialized
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        None
 *
 * Note:            None
 ********************************************************************/
void BerkeleyTCPServerDemo(void)
{
    static SOCKET bsdServerSocket;   
    static SOCKET ClientSock[MAX_CLIENT];
    struct sockaddr_in addr;
    struct sockaddr_in addRemote;
    int addrlen = sizeof(struct sockaddr_in);
    static char RecieveBuffer[198];
    static char SendBuffer[52];
    int length;
    int SocketIndex;
    static enum
    {
	    BSD_INIT = 0,
        BSD_CREATE_SOCKET,
        BSD_BIND,
        BSD_LISTEN,
        BSD_OPERATION
    } BSDServerState = BSD_INIT;
    
    if (bsdServerSocket == INVALID_SOCKET)
    {
    	BSDServerState = BSD_INIT;
    }

    switch(BSDServerState)
    {
	    case BSD_INIT:
        	// Initialize all client socket handles so that we don't process 
        	// them in the BSD_OPERATION state
        	for(SocketIndex = 0; SocketIndex < MAX_CLIENT; SocketIndex++)
        		ClientSock[SocketIndex] = INVALID_SOCKET;
        		
        	BSDServerState = BSD_CREATE_SOCKET;
        	// No break needed
	    
        case BSD_CREATE_SOCKET:
            // Create a socket for this server to listen and accept connections on
            bsdServerSocket = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
            if(bsdServerSocket == INVALID_SOCKET)
                return;
            
            BSDServerState = BSD_BIND;
            // No break needed

        case BSD_BIND:
            // Bind socket to a local port
            addr.sin_port = PORTNUM;
            addr.sin_addr.S_un.S_addr = IP_ADDR_ANY;
            if( bind( bsdServerSocket, (struct sockaddr*)&addr, addrlen ) == SOCKET_ERROR )
                return;
            
            BSDServerState = BSD_LISTEN;
            // No break needed
         
      case BSD_LISTEN:
            if(listen(bsdServerSocket, MAX_CLIENT) == 0)
	            BSDServerState = BSD_OPERATION;

			// No break.  If listen() returns SOCKET_ERROR it could be because 
			// MAX_CLIENT is set to too large of a backlog than can be handled 
			// by the underlying TCP socket count (TCP_PURPOSE_BERKELEY_SERVER 
			// type sockets in TCPIPConfig.h).  However, in this case, it is 
			// possible that some of the backlog is still handleable, in which 
			// case we should try to accept() connections anyway and proceed 
			// with normal operation.
         
      case BSD_OPERATION:
            for(SocketIndex=0; SocketIndex<MAX_CLIENT; SocketIndex++)
            {
	            int BufferPosition;
	            // Accept any pending connection requests, assuming we have a place to store the socket descriptor
                if(ClientSock[SocketIndex] == INVALID_SOCKET)
                    ClientSock[SocketIndex] = accept(bsdServerSocket, (struct sockaddr*)&addRemote, &addrlen);
                
                // If this socket is not connected then no need to process anything
                if(ClientSock[SocketIndex] == INVALID_SOCKET)
                	continue;

	            // Check for data in this coket
                length = recv( ClientSock[SocketIndex], RecieveBuffer, sizeof(RecieveBuffer), 0);
         		BufferPosition = 0;
                if( length > 0 )
                {
	                while((length - BufferPosition) >= 3) //Commands are 3 bytes long so discard chunks of less than 3 bytes
	                {
		                int state;
						int CommandCode = RecieveBuffer[BufferPosition++];
						int Param1 = RecieveBuffer[BufferPosition++];
						int Param2 = RecieveBuffer[BufferPosition++];
						
						/*These firt two cases (7 & 10) require a response therefore 
						they are processed here*/
						if (CommandCode == CLOSE_COMMAND) //Close connection
						{
							if (Param1 == 17 && Param2 == 27)  //verification bytes
							{

									closesocket( ClientSock[SocketIndex] );
                   					ClientSock[SocketIndex] = INVALID_SOCKET;
        			        				
					        }	            			
                   			
						} 
						else if (CommandCode == REQUEST_LED_TABLE_COMMAND) //Request for LED Table
						{
						
							if (Param1 == 110 && Param2 == 210)  //verification bytes
							{
								int SendLength;								
								SendLength = BuildLEDTableResponse(SendBuffer);
								send(ClientSock[SocketIndex], SendBuffer, SendLength, 0);
								
							}
						}
						else if (CommandCode == REQUEST_POWER_COMMAND)
						{
							SendBuffer[0] = REQUEST_POWER_COMMAND; //Response Code
							SendBuffer[1] = (BYTE) (VoltageMeasurement >> 8); //High Byte
							SendBuffer[2] = (BYTE) (VoltageMeasurement); //Low Byte
							SendBuffer[3] = (BYTE) (CurrentMeasurement >> 8); //High Byte
							SendBuffer[4] = (BYTE) (CurrentMeasurement); //Low Byte
							send(ClientSock[SocketIndex], SendBuffer, 5, 0);
						}	
						else //Send it to the LEDCommands module for processing
						{
							LED_COMMAND command;
							command.commandcode = CommandCode;
							command.param[0] = Param1;
							command.param[1] = Param2;
							ProcessLEDCommand(&command);
							
						}	
					}
                }
                else if( length < 0 )
                {
                    closesocket( ClientSock[SocketIndex] );
                    ClientSock[SocketIndex] = INVALID_SOCKET;
                }
            }
            break;
         
        default:
            return;
    }
    return;
}


