#ifndef __HARDWARE_PROFILE_H
#define __HARDWARE_PROFILE_H

#include "GenericTypeDefs.h"
#include "Compiler.h"


//Set configuration fuses.
#if defined(THIS_IS_STACK_APPLICATION)
	#pragma config WDTEN = ON, OSC = INTOSCPLL, SOSCSEL = DIG, RTCOSC = INTOSCREF, ADCSEL = BIT12, PLLDIV = 2, CFGPLLEN = ON, PLLSEL = PLL96
	#pragma config CLKOEC = OFF, DSWDTOSC = INTOSCREF, WDTPS = 8192

	#if defined(__EXTENDED18__)
		#pragma config XINST=ON
	#else
		#pragma config XINST=OFF
	#endif
#endif



// Clock frequency value.
// This value is used to calculate Tick Counter value

#define GetSystemClock()		(48000000ul)      // Hz
#define GetInstructionClock()	(GetSystemClock()/4)
#define GetPeripheralClock()	GetInstructionClock()



#define ADCON2 ADCON1

// Hardware mappings

#define VOLTAGE_CHANNEL 3
#define CURRENT_CHANNEL 2
#define VOLTAGE_PIN 3
#define CURRENT_PIN 2
#define ADC_TRIS TRISA

#define VPP_ON_TRIS TRISAbits.TRISA5
#define VPP_ON LATAbits.LATA5

#define SCL_TRIS TRISBbits.TRISB4
#define SDA_TRIS TRISBbits.TRISB5





/***************Configure LCD*******************************/

/* Interface type 8-bit or 4-bit
 * For 8-bit operation uncomment the #define BIT8
 */
/* #define BIT8 */

/* When in 4-bit interface define if the data is in the upper
 * or lower nibble.  For lower nibble, comment the #define UPPER
 */
/* #define UPPER */

/* DATA_PORT defines the port to which the LCD data lines are connected */
 #define DATA_PORT      		PORTB
 #define TRIS_DATA_PORT 		TRISB

/* CTRL_PORT defines the port where the control lines are connected.
 * These are just samples, change to match your application.
 */
 #define RW_PIN   LATCbits.LATC7   		/* PORT for RW */
 #define TRIS_RW  TRISCbits.TRISC7    	/* TRIS for RW */

 #define RS_PIN   LATCbits.LATC0   		/* PORT for RS */
 #define TRIS_RS  TRISCbits.TRISC0    	/* TRIS for RS */

 #define E_PIN    LATCbits.LATC1  		/* PORT for D  */
 #define TRIS_E   TRISCbits.TRISC1    	/* TRIS for E  */
	
#define USE_LCD



/********************Configure PHY*************************/


//	// ENC424J600/624J600 Fast 100Mbps Ethernet PICtail Plus defines
#define ENC100_INTERFACE_MODE			0		// Uncomment this line to use the ENC424J600/624J600 Ethernet controller (SPI mode) or comment it out to use some other network controller
//	
//	// ENC100_MDIX, ENC100_POR, and ENC100_INT are all optional.  Simply leave 
//	// them commented out if you don't have such a hardware feature on your 
//	// board.
//	#define ENC100_MDIX_TRIS				(TRISBbits.TRISB4)
//	#define ENC100_MDIX_IO					(LATBbits.LATB4)
//	#define ENC100_POR_TRIS					(TRISBbits.TRISB5)
//	#define ENC100_POR_IO					(LATBbits.LATB5)
//	#define ENC100_INT_TRIS					(TRISCbits.TRISC7)
//	#define ENC100_INT_IO					(PORTCbits.RC7)
//
//	// ENC424J600/624J600 SPI pinout
	#define ENC100_CS_TRIS					(TRISCbits.TRISC6)
	#define ENC100_CS_IO					(LATCbits.LATC6)
	#define ENC100_SO_WR_B0SEL_EN_TRIS		(TRISCbits.TRISC4)	// NOTE: SO is ENC624J600 Serial Out, which needs to connect to the PIC SDI pin for SPI mode
	#define ENC100_SO_WR_B0SEL_EN_IO		(PORTCbits.RC4)
	#define ENC100_SI_RD_RW_TRIS			(TRISCbits.TRISC5)	// NOTE: SI is ENC624J600 Serial In, which needs to connect to the PIC SDO pin for SPI mode
	#define ENC100_SI_RD_RW_IO				(LATCbits.LATC5)
	#define ENC100_SCK_AL_TRIS				(TRISCbits.TRISC3)
	#define ENC100_SCK_AL_IO				(PORTCbits.RC3)		// NOTE: This must be the PORT, not the LATch like it is for the PSP interface.
//
//	// ENC424J600/624J600 SPI SFR register selection (controls which SPI 
//	// peripheral to use on PICs with multiple SPI peripherals).
//	#define ENC100_ISR_ENABLE		(INTCON3bits.INT2IE)
//	#define ENC100_ISR_FLAG			(INTCON3bits.INT2IF)
//	#define ENC100_ISR_POLARITY		(INTCON2bits.INTEDG2)
//	#define ENC100_ISR_PRIORITY		(INTCON3bits.INT2IP)
	#define ENC100_SPI_ENABLE		(ENC100_SPISTATbits.SPIEN)
	#define ENC100_SPI_IF			(PIR1bits.SSPIF)
	#define ENC100_SSPBUF			(SSP1BUF)
	#define ENC100_SPISTAT			(SSP1STAT)
	#define ENC100_SPISTATbits		(SSP1STATbits)
	#define ENC100_SPICON1			(SSP1CON1)
	#define ENC100_SPICON1bits		(SSP1CON1bits)
	#define ENC100_SPICON2			(SSP1CON2)


/********************Configure Memory*************************/


/*EEPROM is I2C in this app not SPI */

/*

	// 25LC256 I/O pins
	#define EEPROM_CS_TRIS		(TRISAbits.TRISA3)
	#define EEPROM_CS_IO		(LATAbits.LATA3)
	#define EEPROM_SCK_TRIS		(TRISCbits.TRISC3)
	#define EEPROM_SDI_TRIS		(TRISCbits.TRISC4)
	#define EEPROM_SDO_TRIS		(TRISCbits.TRISC5)
	#define EEPROM_SPI_IF		(PIR1bits.SSPIF)
	#define EEPROM_SSPBUF		(SSP1BUF)
	#define EEPROM_SPICON1		(SSP1CON1)
	#define EEPROM_SPICON1bits	(SSP1CON1bits)
	#define EEPROM_SPICON2		(SSP1CON2)
	#define EEPROM_SPISTAT		(SSP1STAT)
	#define EEPROM_SPISTATbits	(SSP1STATbits)
*/








#endif
