/************************************************************************
This is a for lowe level communication with the LED boards and the EEPROM.
Some functions are adapted from the Microchip C18 Periphreal library.
************************************************************************/

#include "HardwareProfile.h"
#include "I2CComm.h"
#include "delays.h"
#include "LEDCommands.h"



void I2CInit(void)
{
	/*set up I2C pins*/
	
	//T0CONbits.TMR0ON = 0;
	
	SCL_TRIS = 1;
	SDA_TRIS = 1;
	
	
	
	/*Set I2C to Master Mode */
	I2C_SSPCON = 0b1000;
	
	I2C_SSPSTATbits.SMP = 1; //Slew rate control is disabled
	I2C_SSPSTATbits.CKE = 0; //Disable SMBus specifics
	
	/*Set clock to about 50Khz Clock = FOSC/(4 * (SSPxADD + 1)) */
	//SSP2ADD = 239;
	I2C_SSPADD = 127;
	
	I2C_SSPCONbits.SSPEN = 1; //Enable I2C Port
	
}



signed char putsI2C( unsigned char *wrptr )
{
   unsigned char temp;  
	while ( *wrptr )                 // transmit data until null character 
  {
    if ( I2C_SSPCONbits.SSPM3 )      // if Master transmitter then execute the following
    {  
	   temp = putcI2C ( *wrptr );
	   if (temp ) return ( temp );            // return with write collision error
     
    }

    else                           // else Slave transmitter
    {

      I2C_SSPIF = 0;         // reset SSP2IF bit

      I2C_SSPBUF = *wrptr;            // load SSP2BUF with new data
      I2C_SSPCONbits.CKP = 1;        // release clock line 

      while ( !I2C_SSPIF );  // wait until ninth clock pulse received
 

      if ( ( I2C_SSPCONbits.CKP ) && ( !I2C_SSPSTATbits.BF ) )// if R/W=0 and BF=0, NOT ACK was received
      {
        return ( -2 );             // terminate PutsI2C2() function
      }
    }

  wrptr ++;                        // increment pointer

  }                                // continue data writes until null character

  return ( 0 );
}

unsigned char WriteI2C( unsigned char data_out )
{
  I2C_SSPBUF = data_out;           // write single byte to SSP2BUF
  if ( I2C_SSPCONbits.WCOL )      // test if write collision occurred
    return ( -1 );              // if WCOL bit is set return negative #
  else
  {
	if( ((I2C_SSPCON&0x0F)!=0x08) && ((I2C_SSPCON&0x0F)!=0x0B) )	//slave mode only
	{

      if ( ( !I2C_SSPSTATbits.R_W ) && ( !I2C_SSPSTATbits.BF ) )// if R/W=0 and BF=0, NOT ACK was received

      {
        return ( -2 );             // return NACK
      }
	  else return(0);				// return ACK
	
	}  
    else if( ((I2C_SSPCON&0x0F)==0x08) || ((I2C_SSPCON&0x0F)==0x0B) )	//master mode only
	{
	    while( I2C_SSPSTATbits.BF );   // wait until write cycle is complete     
	    IdleI2C();                  // ensure module is idle
	    if ( I2C_SSPCON2bits.ACKSTAT ) // test for ACK condition received
	    	 return ( -2 );				//return NACK
		else 
		{
		return ( 0 );   			//return ACK
		}
	}
	
  }
}

unsigned char ReadI2C( void )
{
	long timeout=0;

  I2C_SSPCON2bits.RCEN = 1;           // enable master for 1 byte reception
  while ( !SSP2STATbits.BF)      // wait until byte received  
  {
	 timeout++;
	 if (timeout > 100000)
	 {
		 //break;
	 } 
  } 
 
  return ( I2C_SSPBUF );              // return with read byte 
}
unsigned char ReadStringI2C2( unsigned char *rdptr, unsigned char length )
{
		long timeout=0;
    while ( length -- )            // perform getcI2C2() for 'length' number of bytes
    {
      *rdptr++ = ReadI2C();       // save byte received
      while ( I2C_SSPCON2bits.RCEN ); // check that receive sequence is over    


	if( ((I2C_SSPCON&0x0F)==0x08) || ((I2C_SSPCON&0x0F)==0x0B) )	//master mode only
	{	
      if ( length )               // test if 'length' bytes have been read
      {
        I2C_SSPCON2bits.ACKDT = 0;    // set acknowledge bit state for ACK        
        I2C_SSPCON2bits.ACKEN = 1;    // initiate bus acknowledge sequence
        while ( SSP2CON2bits.ACKEN ) // wait until ACK sequence is over 
        {
	        timeout++;
			 if (timeout > 100000)
			 {
				 //break;
			 } 
	    } 
      } 
	} 
	  
    }
    return ( 0 );                  // last byte received so don't send ACK      
}
/*Write a single byte to EEPROM. There is no multiwrite
function because multi-byte writes must only be performed on
page boundries. Since EEPROM writing is infrequent in this
application. The performance improvement was not worth the
development hassle.*/
void EEPROMWrite(int address, BYTE data)
{
	/*Dont try if the bus is not powered because it will
	be held low and the ACK routine will enter an infinite loop
	this is expected to be handled at a higher level so the
	power is not cycled repeatedly during multiple accesses */
	if (isPowerOn())
	{
		StartI2C();
		WriteI2C(EEPROM_ADDRESS);
		WriteI2C((BYTE) (address >> 8));
		WriteI2C((BYTE) (address));
		WriteI2C(data);
		StopI2C(); //EEPROM starts write after the stop signal
		Delay10KTCYx(5); //Wait for write cycle to complete
	}
}	

/*Read a single byte from EEPROM*/
BYTE EEPROMRead(int address)
{
	if (isPowerOn())
	{
		BYTE temp;
		StartI2C();
		WriteI2C(EEPROM_ADDRESS);
		WriteI2C((BYTE) (address >> 8));
		WriteI2C((BYTE) (address));
		RestartI2C();
		WriteI2C(EEPROM_ADDRESS + 1);
		temp = ReadI2C();
		StopI2C() ;
		return temp;
	}
}	

/*Read multiple bytes from EEPROM*/
void EEPROMMultiRead(char * buffer, int address, BYTE count)
{
	if (isPowerOn())
	{
		BYTE i;
		StartI2C();
		WriteI2C(EEPROM_ADDRESS);
		WriteI2C((BYTE) (address >> 8));
		WriteI2C((BYTE) (address));
		RestartI2C();
		WriteI2C(EEPROM_ADDRESS + 1);
		ReadStringI2C2(buffer,count);
		/*
		for (i=0; i<count; i++)
		{
			buffer[i] = ReadI2C();
		}*/	
		StopI2C() ;
	}
}	
