#include "HardwareProfile.h"
#include "I2CComm.h"
#include "GenericTypeDefs.h"
#include "Compiler.h"
#include "LEDCommands.h"
#include "TCPIP Stack/Delay.h" 

LED_TABLE LEDTable[LED_TABLE_LENGTH];
BYTE ActiveLEDs = 0;


/*This function interprets and executes incoming
  commands from any of the three interfaces */
void ProcessLEDCommand(LED_COMMAND * command)
{
	switch (command->commandcode)
	{
		case POWER_COMMAND:
				if(command->param[1] == 11) //verification byte
				{
					if(command->param[0]) //Power on command	
					{
						if (!PowerFaultOccurred) //If there was a short, force reset before turning back on
						{
							BYTE i;
							VPP_ON = 1; //Turn power on
							DelayMs(200);
							//Now resume the last state before power down
							for (i = 0; i < ActiveLEDs; i++)
							{
								SetLEDState(LEDTable[i].address, SET_RED_COMMAND, LEDTable[i].red);
								SetLEDState(LEDTable[i].address, SET_GREEN_COMMAND, LEDTable[i].green);
								SetLEDState(LEDTable[i].address, SET_BLUE_COMMAND, LEDTable[i].blue);
							}
						}
					}	
					else //power off command
					{
						//SaveLEDTable(); //Store the LED states in EEPROM
						VPP_ON = 0; //Turn power off
	
						
					}	
				}
		break;
		
		case STORE_LED_COMMAND:
			if(command->param[1] == 210) //verification byte
			{
				StoreLEDAddress(command->param[0]);
			}	
		
		break;
		
		case DELETE_LED_COMMAND:
			if(command->param[1] == 220) //verification byte
			{
				DeleteLEDAddress(command->param[0]);
			}			
		break;
		
		case RESET_COMMAND: //Perform system reset
			if(command->param[0] == 130 && command->param[1] == 230) //verification bytes
			{
				Reset();	
			}	
		break;
		
		case SET_RED_SCALE:
				StartI2C();
				WriteI2C(command->param[0]);
				WriteI2C(SET_RED_SCALE);
				WriteI2C(command->param[1]);
				StopI2C() ;
		break;
		case SET_GREEN_SCALE:

				StartI2C();
				WriteI2C(command->param[0]);
				WriteI2C(SET_GREEN_SCALE);
				WriteI2C(command->param[1]);
				StopI2C() ;
		break;
		case SET_BLUE_SCALE:
				StartI2C();
				WriteI2C(command->param[0]);
				WriteI2C(SET_BLUE_SCALE);
				WriteI2C(command->param[1]);
				StopI2C() ;
		break;
		case SET_RED_SCALE_CONFIRM:
				StartI2C();
				WriteI2C(command->param[0]);
				WriteI2C(SET_RED_SCALE_CONFIRM);
				WriteI2C(command->param[1]);
				StopI2C() ;
		break;
		case SET_GREEN_SCALE_CONFIRM:

				StartI2C();
				WriteI2C(command->param[0]);
				WriteI2C(SET_GREEN_SCALE_CONFIRM);
				WriteI2C(command->param[1]);
				StopI2C() ;
		break;
		case SET_BLUE_SCALE_CONFIRM:
				StartI2C();
				WriteI2C(command->param[0]);
				WriteI2C(SET_BLUE_SCALE_CONFIRM);
				WriteI2C(command->param[1]);
				StopI2C() ;
		break;

		
		default:
			if ((command->commandcode >= SET_RED_COMMAND) && (command->commandcode <= SET_ALL_COMMAND))
			{
				if (isPowerOn())
				{
					SetLEDState(command->param[0], command->commandcode, command->param[1]);
				}
			}	
		break;
	}	
}	
/*This function sends a command over the 2-wire bus
  to set an LED color to a certain value */
void SetLEDState(BYTE address, BYTE color, BYTE val)
{
	signed char LEDIndex; 
	
	//First issue the commands on the bus
	StartI2C();
	WriteI2C(address);
	WriteI2C(color);
	WriteI2C(val);
	StopI2C() ;
	
	//Now make note of the new state
	LEDIndex = LEDAddressToIndex(address);
	if (LEDIndex != -1)
	{
		switch (color)
		{
			case SET_RED_COMMAND:
				LEDTable[LEDIndex].red = val;
			break;
			case SET_GREEN_COMMAND:
				LEDTable[LEDIndex].green = val;
			break;
			case SET_BLUE_COMMAND:
				LEDTable[LEDIndex].blue = val;
			break;
			case SET_ALL_COMMAND:
				LEDTable[LEDIndex].red = val;
				LEDTable[LEDIndex].green = val;
				LEDTable[LEDIndex].blue = val;
			break;
			
		}
	}	
	
}

/*These two functions are planned to be implemented in the
  future. For now, to simplify client software, I am setting
  the LED Board addresses with a serial utility (Bus Pirate) */

void SetLEDAddress(BYTE address)
{
	
}	


void SetLEDRefVoltage(BYTE address, BYTE PWMValue)
{
	
}	

/*This function is for storing the address of a new
  LED Board in the EEPROM so that control interfaces
  are notified about it when they check for status.
  */
void StoreLEDAddress(BYTE address)
{
	
	if (ActiveLEDs < LED_TABLE_LENGTH)
	{
		signed char LEDIndex;	
	    LEDIndex = LEDAddressToIndex(address);
	    if (LEDIndex == -1) //Make sure we only have each address once.
	    {
			LEDTable[ActiveLEDs].address = address;
		}
	}
	ActiveLEDs++;
	SaveLEDTable();
	
}	
/*This function deletes an LED address. We need to move all
  of the table entries after the deleted entry so that no
  holes exist in the table */
void DeleteLEDAddress(BYTE address)
{
	signed char LEDIndex;	
	LEDIndex = LEDAddressToIndex(address);
	if (LEDIndex != -1)
	{
		BYTE i;
		if (LEDIndex != (ActiveLEDs - 1))
		{
			for (i=LEDIndex; i < (ActiveLEDs - 1); i++)
			{
				LEDTable[i].address = LEDTable[i + 1].address;
				LEDTable[i].blue  = LEDTable[i + 1].blue ;
				LEDTable[i].green = LEDTable[i + 1].green;
				LEDTable[i].red = LEDTable[i + 1].red;
			}
			LEDTable[ActiveLEDs - 1].address = 0;
			LEDTable[ActiveLEDs - 1].red = 0;
			LEDTable[ActiveLEDs - 1].blue = 0;
			LEDTable[ActiveLEDs - 1].green = 0;
			
		}
		else
		{
			LEDTable[LEDIndex].address = 0;
			LEDTable[LEDIndex].blue = 0;
			LEDTable[LEDIndex].green = 0;
			LEDTable[LEDIndex].red = 0;
		}	
		ActiveLEDs--;
		SaveLEDTable();	
	}	
}	

/*This function loads previously stored information about the LED
  boards connected the controller from the EEProm */
void LoadLEDTable(void)
{
	BYTE tempTable[sizeof(LEDTable) + 1]; //temp buffer for the bytes read from eeprom
	BYTE i;
	BOOL IsLoadingDone = 0;
	BOOL ONState = isPowerOn();
	if (!ONState) //Bus power has to be on to access EEPROM
	{
		TempPowerOn(); //Turn power on
		DelayMs(200);
	}	
	EEPROMMultiRead(tempTable, LED_TABLE_EEPROM_START_ADDRESS, sizeof(LEDTable)); //read the data from eeprom
	for (i = 0; i < LED_TABLE_LENGTH; i++)
	{
		int StartIndex = (i * sizeof(LEDTable[0]));
		if (IsLoadingDone) //If we are done fill the rest of the table with zeros
		{
			LEDTable[i].address = 0;
			LEDTable[i].red = 0;
			LEDTable[i].blue = 0;
			LEDTable[i].green = 0;
		}
		else
		{	
			LEDTable[i].address = tempTable[StartIndex];
			LEDTable[i].red = tempTable[StartIndex + 1];
			LEDTable[i].blue = tempTable[StartIndex + 2];
			LEDTable[i].green = tempTable[StartIndex + 3];
			if (LEDTable[i].address == 0 || LEDTable[i].address == 255 || (LEDTable[i].address % 2) != 0)
			{
				LEDTable[i].address = 0;
				LEDTable[i].red = 0;
				LEDTable[i].blue = 0;
				LEDTable[i].green = 0;
				IsLoadingDone = 1; //If we get a bad address, assume the previous was the last
				ActiveLEDs = i; //indicate the total number of LEDs	
			}	
		}
		
	}
	if (!IsLoadingDone) //If we never hit the end the table is full
	{
			ActiveLEDs = LED_TABLE_LENGTH;
	}	

	VPP_ON = ONState;
	
}
/*This saves the entire LED table to the EEPROM*/
void SaveLEDTable(void)
{
	int i;
	BOOL ONState = isPowerOn();
	BYTE * PtrLEDTable = &(LEDTable[0].address);
	
	if (!ONState)
	{
		TempPowerOn(); //Turn power on
		DelayMs(200);
	}	
	
	for(i=0; i < sizeof(LEDTable); i++) //Save all bytes to EEPROM
	{
		EEPROMWrite(LED_TABLE_EEPROM_START_ADDRESS + i, *PtrLEDTable);
		PtrLEDTable++;
	}
	VPP_ON = ONState;
}
/*This saves a single entry of the LED table to the EEPROM*/
void SaveLEDTableEntry(BYTE EntryIndex)
{
	int i;
	BOOL ONState = isPowerOn();
	BYTE * PtrLEDTable = &(LEDTable[EntryIndex].address);
	
	if (!ONState)
	{
		TempPowerOn(); //Turn power on
		DelayMs(200);
	}	
	
	for(i=0; i < sizeof(LEDTable[0]); i++)
	{
		EEPROMWrite((sizeof(LEDTable[0]) * EntryIndex) + LED_TABLE_EEPROM_START_ADDRESS + i, *PtrLEDTable);
		PtrLEDTable++;
	}
	VPP_ON = ONState;
}
/*This zeroes out the LED table in the EEPROM */
void ClearLEDTable(void)
{
	int i;
	BOOL ONState = isPowerOn();
	if (!ONState)
	{
		TempPowerOn(); //Turn power on
		DelayMs(200);
	}	
	for(i=0; i < sizeof(LEDTable); i++)
	{
		EEPROMWrite(LED_TABLE_EEPROM_START_ADDRESS + i, 0);
	}	
	VPP_ON = ONState;
}		
/*This is a test function to do the initial 
set-up for the controller*/
void LEDTableTestSetup(void)
{
	BOOL ONState = isPowerOn();
	if (!ONState)
	{
		TempPowerOn(); //Turn power on
		DelayMs(200);
	}	
	ClearLEDTable();
	EEPROMWrite(LED_TABLE_EEPROM_START_ADDRESS, 48);
	EEPROMWrite(LED_TABLE_EEPROM_START_ADDRESS + 4, 50);
	VPP_ON = ONState;
}		

/*This function gets the index in the LEDTable array
  using the 2-wirw bus address */

signed char LEDAddressToIndex(BYTE address)
{
	signed char rIndex = -1;
	BYTE i;
	
	for (i=0; i < ActiveLEDs; i++)
	{
		if (LEDTable[i].address == address)
		{
			rIndex = i;
		}	
	}	 
	return rIndex;
}	

/*This function fills a byte array that will be sent
  when a client application requests
  the LED table. It returns the length */
int BuildLEDTableResponse(BYTE * buffer)
{
	BYTE BufferCount;
	BYTE *PtrLEDTable;
	int SendSuccess;
	int i;
	
	
	
	buffer[0] = REQUEST_LED_TABLE_COMMAND; //response code
	buffer[1] = isPowerOn(); //Main power state
	buffer[2] = ActiveLEDs; //number of leds
	BufferCount = 3;
	
	PtrLEDTable = &(LEDTable[0].address);
	for(i=0; i < (sizeof(LEDTable[0]) * ActiveLEDs); i++)
	{
		buffer[BufferCount] = *PtrLEDTable;
		PtrLEDTable++;
		BufferCount++;
	}
	return BufferCount;
}
