/*
 * This macro uniquely defines this file as the main entry point.
 * There should only be one such definition in the entire project,
 * and this file must define the AppConfig variable as described below.
 */
#define THIS_IS_STACK_APPLICATION

/*********************************************************************/
/*****Begin Include Files Section*********/


//Include all headers for any enabled TCPIP Stack functions
#include "TCPIP Stack/TCPIP.h" 

#include "I2CComm.h"
#include "configapp.h"
#include "GenericTypeDefs.h"
#include "Compiler.h"
#include "WirelessProtocols/MCHP_API.h"
#include "string.h"
#include "delays.h"
#include "LEDCommands.h"


/*****End Include Files Section*********/
/*********************************************************************/


/*********************************************************************/
/*****Begin Defines*********/

#define DisplayROMString(a) strcpypgm2ram(LCDText, a); LCDUpdate()

#define ADC_CURRENT_FAULT_THRESHOLD 2000 //~1.6A
#define CURRENT_FAULT_MAX_COUNT 3

#define ADC_VOLTAGE_FAULT_THRESHOLD 2700 // ~38 volts
#define VOLTAGE_FAULT_MAX_COUNT 3

	


/*****End Defines*********/
/*********************************************************************/




/*********************************************************************/
/*****Begin Function Prototypes*********/



void MiWiTimerTick(void);
static void InitAppConfig(void);
static void InitHardware(void);
void DisplayIPValue(IP_ADDR IPVal);
void BerkeleyTCPServerDemo(void);
void P2PServerTaks(void);
void P2PServerTick(void);
void high_isr (void);
void LowISR(void);




/*****End Function Prototypes*********/
/*********************************************************************/




/*********************************************************************/
/*****Begin Static Variables*********/

// Declare AppConfig structure and some other supporting stack variables
APP_CONFIG AppConfig;

BOOL LCDUpdated = 0;

unsigned int VoltageResultsCount = 0;
unsigned long TotalVoltageResult = 0;
unsigned int CurrentResultsCount = 0;
unsigned long TotalCurrentResult = 0;
unsigned int VoltageMeasurement = 0;
unsigned int CurrentMeasurement = 0;
BYTE P2PInitialized = 0;
BYTE CurrentFaultCount = 0;
BYTE VoltageFaultCount = 0;
BYTE PowerFaultOccurred = 0;




/*****End Static Variables*********/
/*********************************************************************/


/*********************************************************************/
/*****Begin Constants*********/

 const char rom INIT_TCP_STRING[] = "Initializing    TCP/IP";
 const char rom SEEKING__MIWI_STRING[] = "Seeking         MiWi Channel";

/*****End Constants*********/
/*********************************************************************/


/*********************************************************************/
/*****Begin Interrupt Service Routines *********/

#pragma interruptlow LowISR
void LowISR(void)
{
		
    if(INTCONbits.TMR0IF)
    {
		// Increment internal high tick counter
		TickUpdate();
		MiWiTimerTick();

		// Reset interrupt flag
        INTCONbits.TMR0IF = 0;
    }
    
    
    if (PIR1bits.ADIF) //is it an ADC interrupt
    {
    	//These are static to save MCU cycles
    	static unsigned char digit = 0;
   		static unsigned long remainder = 0;
   		static unsigned char lcdPos= 0;
    		
    	static unsigned int adcResult;
    	adcResult = (((unsigned int) ADRESH) << 8) |  ((unsigned int) ADRESL);

    	
    	PIR1bits.ADIF = 0; //Clear A/D interrupt flag
    	if ((ADCON0 & (0b1111 << 2)) == (VOLTAGE_CHANNEL << 2))
    	{

    			
	    	TotalVoltageResult += ((unsigned long) adcResult);
	    	VoltageResultsCount++;
	    	
	    	if (adcResult >= ADC_VOLTAGE_FAULT_THRESHOLD)
		    {
			    VoltageFaultCount++;
			    if (VoltageFaultCount >= VOLTAGE_FAULT_MAX_COUNT)
			    {
				    VPP_ON = 0;
				    PowerFaultOccurred = 1;
				} 
			} 
			else
			{
				CurrentFaultCount = 0;
			}	
	    	
	    	
	    	if (VoltageResultsCount > 255)
	    	{
	            digit = 0;
    		    remainder = 0;
    		    lcdPos= 0;
		    	TotalVoltageResult = (TotalVoltageResult >> 8);
	    		lcdPos = 0;
	    		/*transfer function for voltage is ADC Result / 69.252 */
	    		/*implement some simple decimal-based fixed point math to calculate voltage */
	    		TotalVoltageResult *= 100000;
	    		remainder = TotalVoltageResult % 69252;
	    		TotalVoltageResult /= 69252;
	    		if (remainder > (69252 / 2)) //round it up if warranted
	    		{
	    			TotalVoltageResult++;
	    		}
	    		VoltageMeasurement = (unsigned int) TotalVoltageResult;
	    		
	    		/*Now that we have the results, convert it into a
	    		  displayable string. This is done manually because
	    		  it occurs often and is faster than sprintf */
	    		
	    		digit = (unsigned char) (TotalVoltageResult / 1000);
		    	LCDText[lcdPos] = (digit + 48);
		    	TotalVoltageResult = TotalVoltageResult % 1000;
		    	
		    	lcdPos++;
		    	digit = (unsigned char) (TotalVoltageResult / 100);
		    	LCDText[lcdPos] = (digit + 48);
		    	TotalVoltageResult = TotalVoltageResult % 100;
		    	
		    	lcdPos++;
		        LCDText[lcdPos] = 46; //Code for period(.)
		    	
		    	lcdPos++;
		    	digit = (unsigned char) (TotalVoltageResult / 10);
		    	LCDText[lcdPos] = (digit + 48);
		    	TotalVoltageResult = TotalVoltageResult % 10;
		    	
		    	/* This is code to add a second decimal digit
		    	lcdPos++;
		    	digit = (unsigned char) (TotalVoltageResult);
		    	LCDText[lcdPos] = (digit + 48);	
		    	*/ 
		    	
		    	lcdPos++;
		    	LCDText[lcdPos] = 86; //Code for V
		    	
		    	LCDUpdated = 1;
		    	VoltageResultsCount = 0;
		    	TotalVoltageResult = 0;
	    	}
	    	ADCON0 &= ~(0b1111 << 2); ADCON0 |= (CURRENT_CHANNEL << 2); //Set channel to current measurment.
	    	
    	}
    	else if ((ADCON0 & (0b1111 << 2)) == (CURRENT_CHANNEL << 2))
    	{
	        TotalCurrentResult += adcResult;
	    	CurrentResultsCount++;
	    	
		    if (adcResult >= ADC_CURRENT_FAULT_THRESHOLD)
		    {
			    CurrentFaultCount++;
			    if (CurrentFaultCount >= CURRENT_FAULT_MAX_COUNT)
			    {
				    VPP_ON = 0;
				    PowerFaultOccurred = 1;
				} 
			} 
			else
			{
				CurrentFaultCount = 0;
			}	

	    	
	    	if (CurrentResultsCount > 63)
	    	{
	            digit = 0;
    		    remainder = 0;
    		    lcdPos= 6; //Set LCD cursor
		    	TotalCurrentResult = TotalCurrentResult >> 6;
	    		/*transfer function for current is ADC Result / 1241.2 */
	    		/*implement some simple decimal-based fixed point math to calculate voltage */
	    		TotalCurrentResult *= 10000UL;
	    		remainder = TotalCurrentResult % 12412UL;
	    		TotalCurrentResult /= 12412UL;
	    		if (remainder > (12412UL / 2)) //round it up if warranted
	    		{
	    			TotalCurrentResult++;
	    		}
	    		CurrentMeasurement = (unsigned int) TotalCurrentResult;
	    		digit = (unsigned char) (TotalCurrentResult / 1000UL);
		    	LCDText[lcdPos] = (digit + 48);
		    	TotalCurrentResult = TotalCurrentResult % 1000UL;
		    	
		        lcdPos++;
		        LCDText[lcdPos] = 46; //Code for period(.)
		    	
		    	lcdPos++;
		    	digit = (unsigned char) (TotalCurrentResult / 100UL);
		    	LCDText[lcdPos] = (digit + 48);
		    	TotalCurrentResult = TotalCurrentResult % 100UL;
		    	

		    	
		    	lcdPos++;
		    	digit = (unsigned char) (TotalCurrentResult / 10UL);
		    	LCDText[lcdPos] = (digit + 48);
		    	TotalCurrentResult = TotalCurrentResult % 10UL;
		    	
		    	/* This is code to add a third decimal digit
		    	lcdPos++;
		    	digit = (unsigned char) (TotalCurrentResult);
		    	LCDText[lcdPos] = (digit + 48);	 
		    	*/
		    	
		    	lcdPos++;
		    	LCDText[lcdPos] = 65; //Code for A
		    	
		    	LCDUpdated = 1;
		    	CurrentResultsCount = 0;
		    	TotalCurrentResult = 0;
	    	}
    		ADCON0 &= ~(0b1111 << 2); ADCON0 |= (VOLTAGE_CHANNEL << 2); //Set channel to voltage measurment.
 
    	}


    	
    	
    	
    } //end ADC interuppt
    
    if (PIR3bits.RTCCIF) //RTCC Interuppt (every 10 seconds)
    {
		PIR3bits.RTCCIF = 0; //clear the flag
		if (P2PInitialized)
		{
			P2PServerTick();	
		}	
		    
	} 
}


/*
High Priority interrupt routine is located in  CustomMRF24J40.c
*/
#pragma code lowVector=0x18
void LowVector(void){_asm goto LowISR _endasm}
#pragma code // Return to default code section



	 



/*****End Interrupt Service Routines *********/
/*********************************************************************/


/*********************************************************************/
/*****Begin My Function Definitions *********/

void main(void)
{
	static DWORD t = 0;
	static DWORD dwLastIP = 0;
	extern LED_TABLE LEDTable[LED_TABLE_LENGTH];
	BYTE i;
	
	/***************One Time Stuff**********************/

	InitHardware();
	
	DelayMs(100);
	LCDInit();
	DelayMs(100);
	
	TickInit();  //Init Ticker for TCP/IP
	
	DisplayROMString(INIT_TCP_STRING);

	
	MPFSInit(); //MPFS is Microchip's library from web page storage
	
	InitAppConfig(); //Init TCP/IP stack config
	
	StackInit(); //Init core TCP/IP stuff
	
	//Reset Watchdog
	ClrWdt();
	
	I2CInit(); //Init 2-wire for communicating with the LED boards
	//LEDTableTestSetup();

	LoadLEDTable(); //Load stored information about the connected LED boards



	//Start up MiWi interface
	MiApp_ProtocolInit(FALSE);
    MiApp_ConnectionMode(ENABLE_ALL_CONN);
    
    DisplayROMString(SEEKING__MIWI_STRING);

	//Check for the best channel, then start the MiWi PAN
	MiApp_StartConnection(START_CONN_ENERGY_SCN, 8, 0xFFFFFFFF);
	
	//Display the new channel number in the upper 
	//right of the LCD.
	LCDErase();
	LCDText[14] = (currentChannel / 10) + 48;
	LCDText[15] = (currentChannel % 10) + 48;
	LCDUpdate();
	P2PInitialized = 1;
	/********************End One Time Stuff*****************************/

	
	//Here is the main multi-tasking loop
	while(1)
    {
		static int wCount = 0;
		static BYTE wBar = 1;
		int i = 0;
		
		//Reset Watchdog
		ClrWdt();
		
	    //Tend to P2P Miwi Connection
		P2PServerTaks();
       
        // This task performs normal TCP/IP tasks including checking
        // for incoming packet, type of packet and calling
        // appropriate stack entity to process it.
               
        StackTask();

        // This tasks invokes each of the core TCP/IP stack 
        // application tasks
        StackApplications();
        		 

		//This is to handle any incoming commands from the PC App
		BerkeleyTCPServerDemo();
	


        // If the local IP address has changed (ex: due to DHCP lease change)
        // write the new IP address to the LCD display, UART, and Announce 
        // service
		if(dwLastIP != AppConfig.MyIPAddr.Val)
		{
			dwLastIP = AppConfig.MyIPAddr.Val;
			

			DisplayIPValue(AppConfig.MyIPAddr);


			#if defined(STACK_USE_ANNOUNCE)
				AnnounceIP();
			#endif
		}
		if (LCDUpdated)
		{
			LCDUpdate();
			LCDUpdated = 0;
		}
		
	

	}
} //End main function

int P2PAddressToIndex(BYTE * RecAddress)
{
	int returnVal = -1;
	BYTE i;
	BYTE k;
	for (i=0; i < CONNECTION_SIZE; i++)
	{
		if (ConnectionTable[i].status.bits.isValid)
		{
			BYTE AddIsEqual = 1;
			for (k=0; k < MY_ADDRESS_LENGTH; k++)
			{
				if (ConnectionTable[i].Address[k] != RecAddress[k])
				{
					AddIsEqual = 0;
				}	
			}	
			if (AddIsEqual)
			{
				returnVal = i;
			}	
		}	
	}	
	return returnVal;
}	

void InitHardware(void)
{
	/******************Set Ititial State of IO Pins***************************/
	
	//Deassert both CS lines to avoid conflict
	PHY_CS = 1;
	PHY_CS_TRIS = 0;
	
	ENC100_CS_IO = 1;
	ENC100_CS_TRIS	= 0;
	
	//Start with all pins set as digital then
	//change as neccessary later
	ANCON0 = 0xFF;
	ANCON1 |= 0xFF;
	
	//Disable the RTCC clock output so we can
	//use RB1 as GPIO
	RTCCFGbits.RTCOE = 0;
	RTCCFGbits.RTCWREN = 1;
	RTCCFGbits.RTCEN = 0;
	
	//Disable reference clock output for RB2
	REFOCONbits.ROON = 0;
	
	//Set proper direction of SPI pins
	ENC100_SCK_AL_TRIS = 0;
	ENC100_SI_RD_RW_TRIS = 0;
	ENC100_SO_WR_B0SEL_EN_TRIS = 1;
	
	RF_INT_TRIS = 1; //Set RF Int as input
	
	//Start with power off to the lights
	VPP_ON = 0;
	VPP_ON_TRIS = 0;
	
	
	/*********************************************************************/
	
	
	/******************Periphrial Pin Select***************************/
	
	PPSCON = 0; //Unlock PPS
	RF_INT_PPS = 2; //Assign External Interrupt 2 to Pin for MiWi
	PPSCON = 1; //Lock PPS
	
	
	/*********************************************************************/
	
	
		
	/******************Set Up ADC***************************/
	ADCON0bits.VCFG1 = 0; //Negative reference is AVSS
	ADCON0bits.VCFG0 = 0; //Positive reference is AVDD
	ADCON1bits.ADFM = 1; //result is right justified

	ADCON1 &= ~(0b111 << 3); ADCON1 |= (0b110 << 3); //Aquisition Time is 16 TAD
	ADCON1 &= ~(0b111); ADCON1 |= (0b110); //A/D clock is FOSC/64.
	
	ADC_TRIS |= ((1 << VOLTAGE_PIN) | (1 << CURRENT_PIN)); //set digital pins as inputs
	ANCON0 &= ~((1 << VOLTAGE_CHANNEL) | (1 << CURRENT_CHANNEL)); //Set analog pins as analog inputs
	
	ADCON0 &= ~(0b1111 << 2); ADCON0 |= (VOLTAGE_CHANNEL << 2); //Set initial channel to voltage measurment.
	
	ADCON0bits.ADON =1; //AD Converter is enabled
	
	ADCTRIG = 0b10; //Timer 1 Overflow triggers the ADC
	
	/*********************************************************/
	

	/******************Calibrate ADC***************************/
	ADCON1bits.ADCAL = 1; //Set calibration bit.
	ADCON0bits.GO = 1; //Start dummy conversion
	while (ADCON0bits.GO); //Wait for it to finish
	ADCON1bits.ADCAL = 0; //Clear calibration bit.
	/*********************************************************/
	
	
	/******************Configure Timer 1***************************/
	
		/* configure timer 1 to overflow 732 times per second */

	TMR1H = 0;
	TMR1L = 0; //reset counter registers.
	T1CON = 0b01000111; //Clock is FOSC, Crystal Driver off, 16-bit read/write enabled, timer 1 enabled
	
	
	/*********************************************************/
	
	/******************Configure SPI***************************/
	
	ENC100_SPICON1 = 0;							/* SSPEN = 0 */					\
	ENC100_SPISTAT = 0x40;						/* CKE = 1 (transmit data on rising edge), SMP = 0 (sample at middle of output time) */ \
	ENC100_SPICON1 = 1 | 0x20;	                /* SSPEN = 1, set clock speed */\
	
	
	/*********************************************************/
	
	/******************Set Up RTCC***************************/
	RTCCFGbits.RTCOE = 0;
	ALRMRPT = 1; //Alarm repeat counter
	ALRMCFGbits.CHIME = 1; //Enable chime
	ALRMCFGbits.ALRMEN = 1; //Enable Alarm
	
	/*Chime every 10 seconds */
	ALRMCFGbits.AMASK0 = 0;
	ALRMCFGbits.AMASK1 = 1;
	ALRMCFGbits.AMASK2 = 0;
	ALRMCFGbits.AMASK3 = 0;
	
	//Enable RTCC
/* Disable Interrupt */
   INTCONbits.PEIE = 0;

	_asm movlw 0x55 _endasm
	_asm movwf EECON2,0 _endasm
	_asm movlw 0xAA _endasm
	_asm movwf EECON2,0 _endasm

   RTCCFGbits.RTCWREN=1;  

/* Enable Interrupt */
   INTCONbits.PEIE = 1;
   
	RTCCFGbits.RTCEN = 1;
	
	
	
	/*********************************************************/
	

	/******************Set Up Interrupts***************************/
	
	/*enable RTCC interrupt */
	PIR3bits.RTCCIF = 0;
	PIE3bits.RTCCIE = 1;
	
	IPR1bits.ADIP = 0; //AD interrupt is Low Priority
	PIR1bits.ADIF = 0; //Set ADC Flag low
	PIE1bits.ADIE = 1; //Enable ADC interrupt
	
	INTCON2bits.INTEDG2 = 0; //RF Interrupt is on the falling edge
	
	INTCONbits.TMR0IE = 1; //Enable Timer 0 Interrupt
	
	//Set up Interrupt priorities
	//All low except Miwi
	
	IPR1 = 0;
	IPR2 = 0;
	IPR3 = 0;
	IPR4 = 0;
	IPR5 = 0;
	INTCON2bits.TMR0IP = 0;
	INTCON2bits.INT3IP = 0;
	INTCON2bits.RBIP = 0;
	INTCON3bits.INT1IP = 0;
	
	RFIP = 1;
	
	
	RCONbits.IPEN = 1;		// Enable interrupt priorities
	
	 //Enable Ext Int for MiWi
	 RFIF = 0;
     RFIE = 1;
	
	// Globally Enable Both Interrupt Priorities
    INTCONbits.GIEH = 1;
    INTCONbits.GIEL = 1;
	
	/*********************************************************/
	
	

	


	
	
	
	

	
} //end InitHardware function	


/*****End My Function Definitions *********/
/*********************************************************************/




/*********************************************************************/
/*****Begin Microchip Function Definitions *********/

// Writes an IP address to the LCD display and the UART as available
void DisplayIPValue(IP_ADDR IPVal)
{
//	printf("%u.%u.%u.%u", IPVal.v[0], IPVal.v[1], IPVal.v[2], IPVal.v[3]);
    BYTE IPDigit[4];
	BYTE i;
#ifdef USE_LCD
	BYTE j;
	BYTE LCDPos=16;
#endif

	for(i = 0; i < sizeof(IP_ADDR); i++)
	{
	    uitoa((WORD)IPVal.v[i], IPDigit);


		#ifdef USE_LCD
			for(j = 0; j < strlen((char*)IPDigit); j++)
			{
				LCDText[LCDPos++] = IPDigit[j];
			}
			if(i == sizeof(IP_ADDR)-1)
				break;
			LCDText[LCDPos++] = '.';
		#else
			if(i == sizeof(IP_ADDR)-1)
				break;
		#endif


	}

	#ifdef USE_LCD
		if(LCDPos < 32u)
			LCDText[LCDPos] = 0;
		LCDUpdate();
	#endif
}

/*********************************************************************
 * Function:        void InitAppConfig(void)
 *
 * PreCondition:    MPFSInit() is already called.
 *
 * Input:           None
 *
 * Output:          Write/Read non-volatile config variables.
 *
 * Side Effects:    None
 *
 * Overview:        None
 *
 * Note:            None
 ********************************************************************/
// MAC Address Serialization using a MPLAB PM3 Programmer and 
// Serialized Quick Turn Programming (SQTP). 
// The advantage of using SQTP for programming the MAC Address is it
// allows you to auto-increment the MAC address without recompiling 
// the code for each unit.  To use SQTP, the MAC address must be fixed
// at a specific location in program memory.  Uncomment these two pragmas
// that locate the MAC address at 0x1FFF0.  Syntax below is for MPLAB C 
// Compiler for PIC18 MCUs. Syntax will vary for other compilers.
//#pragma romdata MACROM=0x1FFF0
static ROM BYTE SerializedMACAddress[6] = {MY_DEFAULT_MAC_BYTE1, MY_DEFAULT_MAC_BYTE2, MY_DEFAULT_MAC_BYTE3, MY_DEFAULT_MAC_BYTE4, MY_DEFAULT_MAC_BYTE5, MY_DEFAULT_MAC_BYTE6};
//#pragma romdata

static void InitAppConfig(void)
{
	AppConfig.Flags.bIsDHCPEnabled = TRUE;
	AppConfig.Flags.bInConfigMode = TRUE;
	memcpypgm2ram((void*)&AppConfig.MyMACAddr, (ROM void*)SerializedMACAddress, sizeof(AppConfig.MyMACAddr));
//	{
//		_prog_addressT MACAddressAddress;
//		MACAddressAddress.next = 0x157F8;
//		_memcpy_p2d24((char*)&AppConfig.MyMACAddr, MACAddressAddress, sizeof(AppConfig.MyMACAddr));
//	}
	AppConfig.MyIPAddr.Val = MY_DEFAULT_IP_ADDR_BYTE1 | MY_DEFAULT_IP_ADDR_BYTE2<<8ul | MY_DEFAULT_IP_ADDR_BYTE3<<16ul | MY_DEFAULT_IP_ADDR_BYTE4<<24ul;
	AppConfig.DefaultIPAddr.Val = AppConfig.MyIPAddr.Val;
	AppConfig.MyMask.Val = MY_DEFAULT_MASK_BYTE1 | MY_DEFAULT_MASK_BYTE2<<8ul | MY_DEFAULT_MASK_BYTE3<<16ul | MY_DEFAULT_MASK_BYTE4<<24ul;
	AppConfig.DefaultMask.Val = AppConfig.MyMask.Val;
	AppConfig.MyGateway.Val = MY_DEFAULT_GATE_BYTE1 | MY_DEFAULT_GATE_BYTE2<<8ul | MY_DEFAULT_GATE_BYTE3<<16ul | MY_DEFAULT_GATE_BYTE4<<24ul;
	AppConfig.PrimaryDNSServer.Val = MY_DEFAULT_PRIMARY_DNS_BYTE1 | MY_DEFAULT_PRIMARY_DNS_BYTE2<<8ul  | MY_DEFAULT_PRIMARY_DNS_BYTE3<<16ul  | MY_DEFAULT_PRIMARY_DNS_BYTE4<<24ul;
	AppConfig.SecondaryDNSServer.Val = MY_DEFAULT_SECONDARY_DNS_BYTE1 | MY_DEFAULT_SECONDARY_DNS_BYTE2<<8ul  | MY_DEFAULT_SECONDARY_DNS_BYTE3<<16ul  | MY_DEFAULT_SECONDARY_DNS_BYTE4<<24ul;


	// Load the default NetBIOS Host Name
	memcpypgm2ram(AppConfig.NetBIOSName, (ROM void*)MY_DEFAULT_HOST_NAME, 16);
	FormatNetBIOSName(AppConfig.NetBIOSName);


	#if defined(EEPROM_CS_TRIS)
	{
		BYTE c;
		
	    // When a record is saved, first byte is written as 0x60 to indicate
	    // that a valid record was saved.  Note that older stack versions
		// used 0x57.  This change has been made to so old EEPROM contents
		// will get overwritten.  The AppConfig() structure has been changed,
		// resulting in parameter misalignment if still using old EEPROM
		// contents.
		XEEReadArray(0x0000, &c, 1);
	    if(c == 0x60u)
		    XEEReadArray(0x0001, (BYTE*)&AppConfig, sizeof(AppConfig));
	    else
	        SaveAppConfig();
	}
	#elif defined(SPIFLASH_CS_TRIS)
	{
		BYTE c;
		
		SPIFlashReadArray(0x0000, &c, 1);
		if(c == 0x60u)
			SPIFlashReadArray(0x0001, (BYTE*)&AppConfig, sizeof(AppConfig));
		else
			SaveAppConfig();
	}
	#endif
}

#if defined(EEPROM_CS_TRIS) || defined(SPIFLASH_CS_TRIS)
void SaveAppConfig(void)
{
	// Ensure adequate space has been reserved in non-volatile storage to 
	// store the entire AppConfig structure.  If you get stuck in this while(1) 
	// trap, it means you have a design time misconfiguration in TCPIPConfig.h.
	// You must increase MPFS_RESERVE_BLOCK to allocate more space.
	#if defined(STACK_USE_MPFS) || defined(STACK_USE_MPFS2)
		if(sizeof(AppConfig) > MPFS_RESERVE_BLOCK)
			while(1);
	#endif

	#if defined(EEPROM_CS_TRIS)
	    XEEBeginWrite(0x0000);
	    XEEWrite(0x60);
	    XEEWriteArray((BYTE*)&AppConfig, sizeof(AppConfig));
    #else
	    SPIFlashBeginWrite(0x0000);
	    SPIFlashWrite(0x60);
	    SPIFlashWriteArray((BYTE*)&AppConfig, sizeof(AppConfig));
    #endif
}
#endif


/*****End Microchip Function Definitions *********/
/*********************************************************************/